local Lib = MissionMinderPreLoad;
local DEBUG = Lib.debug;
Lib.callbacks = Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

Lib.tracker = {};
local module = Lib.tracker;

local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");
local COMPLETED = L["completed"]


local SECONDS = 1
local MINUTES = 60 * SECONDS
local HOURS = 60 * MINUTES
local DAYS = 24 * HOURS


local MAX_GARRISON_CACHE = 500
local GARRISON_CACHE_RATE_IN_SECONDS = 10 * MINUTES
local GARRISON_CACHE_WARN_AMOUNT = MAX_GARRISON_CACHE


local TIMES = { DAYS, HOURS, MINUTES}
local TIMESTRS = {INT_SPELL_DURATION_DAYS, INT_SPELL_DURATION_HOURS, INT_SPELL_DURATION_MIN }
function module.secondsToString(seconds)
	local str = ""

	if  ( seconds < MINUTES ) then
		str = str .. string.format(INT_SPELL_DURATION_SEC, seconds);
	else
		local reportOn=2
		for i=1, #TIMES do
			if ( seconds >= TIMES[i] ) then
				if(reportOn <= 0) then
					break;
				end

				if ( str:len() > 0 ) then
					str = str .. " ";
				end

				str = str .. string.format(TIMESTRS[i], seconds / TIMES[i])
				seconds = seconds % TIMES[i]
				reportOn = reportOn - 1
			end
		end
	end

	return str
end


local function getPlayerName()
	return GetUnitName("player") .. "-" .. GetRealmName()
end


function module.estimateGarrisonCache(player)
	local now = time()
	local lastLooted = Lib.Globals.Characters[player].GarrisonCache

	if ( lastLooted ) then
		local cache = math.floor((now - lastLooted) / GARRISON_CACHE_RATE_IN_SECONDS);
		return cache < MAX_GARRISON_CACHE and cache or MAX_GARRISON_CACHE
	else
		return nil
	end
end


local function orderTrackedMissions(track1, track2)
	if ( track1.completed ~= track2.completed ) then
		return track1.completed > track2.completed
	elseif ( track1.lowest ~= track2.lowest ) then
		return track1.lowest < track2.lowest
	else
		return strcmputf8i(track1.name, track2.name) < 0;
	end
end

local function showTrackMissionTooltip()
	local now = time();
	local currentPlayer = getPlayerName()

	local currentPlayerTbl = nil

	local tracking = {}
	local characters = Lib.Globals.Characters
	for player, tbl in pairs(characters) do
		local lowest = nil;
		local completed = 0

		if ( tbl.missions ) then
			for missionID, mission in pairs(tbl.missions) do
				local endDate
				if ( type(mission) == "number" ) then
					endDate = mission
				else
					endDate = mission.endDate
				end

				if ( endDate <= now ) then
					lowest = 0;
					completed = completed + 1
				elseif ( not lowest or endDate - now < lowest ) then
					lowest = endDate - now;
				end
			end
		end

		if ( lowest ) then
			local tbl = {}
			tbl.name = player
			tbl.lowest = lowest
			tbl.completed = completed

			if (player == currentPlayer ) then
				currentPlayerTbl = tbl
			else
				tinsert(tracking, tbl)
			end
		end
	end

	if ( currentPlayerTbl ) then
		if ( currentPlayerTbl.completed > 0 ) then
				GameTooltip:AddLine(currentPlayerTbl.completed .. " " .. COMPLETED, 0, 1, 0);
		else
				GameTooltip:AddLine(module.secondsToString(currentPlayerTbl.lowest), 0.7, 0.7, 0.7);
		end
	end

	-- Mission progress should always show when available
	if ( #tracking > 0 ) then
		GameTooltip:AddLine(" ");
		GameTooltip:AddLine(GARRISON_MISSIONS);
		table.sort(tracking, orderTrackedMissions);
		for i = 1, #tracking do
			if ( tracking[i].completed > 0 ) then
				GameTooltip:AddDoubleLine(tracking[i].name, tracking[i].completed .. " " .. COMPLETED, 1, 1, 1, 0, 1, 0);
			else
				GameTooltip:AddDoubleLine(tracking[i].name, module.secondsToString(tracking[i].lowest), 1, 1, 1, 0.7, 0.7, 0.7);
			end
		end;
	end

	-- Building progress should always show when applicable

	-- if any GarrisonCache is > 356 (less than 1 day from maxing)
	local garrisonCaches = {};
	for player, tbl in pairs(characters) do
		local cache = module.estimateGarrisonCache(player)
		if ( cache and cache >= GARRISON_CACHE_WARN_AMOUNT ) then
			tinsert(garrisonCaches, {
				["character"] = player,
				["amount"] = cache
			});
		end
	end
	if ( #garrisonCaches > 0 ) then
		GameTooltip:AddLine(" ");
		GameTooltip:AddLine(GARRISON_CACHE);
		table.sort(garrisonCaches, sortGarrisonCache)
		for i = 1, #garrisonCaches do
			GameTooltip:AddDoubleLine(garrisonCaches[i].player, garrisonCaches[i].amount, 1, 1, 1, 0, 0, 0);
		end
	end

	-- if any work orders are nearly complete and need to be refilled in less than a day


	GameTooltip:Show()
end

local function trackGarrisonCache(event, quanity)
	local currentPlayer = getPlayerName()
	local estimated = module.estimateGarrisonCache(currentPlayer)
	Lib.Globals.Characters[currentPlayer].GarrisonCache = time()

	if ( estimated ) then
		print("GarrisonCache estimated:", estimated, "actual:", quanity);
	else
		print("GarrisonCache acutal:", quanity);
	end
end

local function debugShipment(self, event, ...)
	local success, _, max, plotID = ...;

	print(self, event, ...)
	local pending = C_Garrison.GetNumPendingShipments();
	print(pending, max, max-pending)

	--local id, name, texPrefix, icon, rank, isBuilding, timeStart, buildTime, canActivate, canUpgrade, isPrebuilt = C_Garrison.GetOwnedBuildingInfoAbbrev(plotID);
	--select(6, C_Garrison.GetPendingShipmentInfo(numPending))
end

local function initialize()
	DEBUG("   |cffffffffpreload-Tracker|r initialized");

	-- Display mission status on GarrisonReport tooltip
	GarrisonLandingPageMinimapButton:HookScript("OnEnter", showTrackMissionTooltip)

	-- Track garrison cache across multiple characters
	Lib.RegisterCallback("tracker", "GARRISON_CACHE_LOOTED", trackGarrisonCache)

	-- Shipment info
	--Lib.RegisterEvent("tracker", "SHIPMENT_CRAFTER_INFO", debugShipment);
end

Lib:RegisterEvent("tracker", "ADDON_LOADED", initialize);
