local Lib = MissionMinder;
local PreLoadLib = MissionMinderPreLoad;
Lib.callbacks = Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

local options = Lib.options;
local DEBUG = Lib.debug;

Lib.tracker = {};
local module = Lib.tracker;

local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");
local L_NO_DATA_AVAILABLE = L["No data available"];
local L_LOADING = LFG_LIST_LOADING

local selectedPlayer = nil
local inprogressMissions = nil

local function getPlayerName()
	return GetUnitName("player") .. "-" .. GetRealmName()
end

local function isCurrentPlayerSelected()
	return getPlayerName() == selectedPlayer
end

local function orderInProgressMissions(mission1, mission2)
	local now = time()

	local mission1Complete = now >= mission1.endDate
	local mission2Complete = now >= mission2.endDate
	if ( mission1Complete ~= mission2Complete ) then
		return mission1Complete
	end

	if ( mission1.iLevel ~= mission2.iLevel ) then
		return mission1.iLevel > mission2.iLevel
	end

	if ( mission1.level ~= mission2.level ) then
		return mission1.level > mission2.level
	end

	return strcmputf8i(mission1.name, mission2.name) < 0;
end

local function loadInProgressMissions()
	if ( isCurrentPlayerSelected() ) then
		return nil
	end

	local tbl = {};
	local missions = PreLoadLib.Globals.Characters[selectedPlayer].missions
	if ( missions ) then
		for missionID, mission in pairs(missions) do
			if ( type(mission) == "table" ) then
				tinsert(tbl, mission);
			end
		end

		table.sort(tbl, orderInProgressMissions)
	end

	return tbl
end

local function selectCharacter(player)
	DEBUG("selectedPlayer = %s", player)

	selectedPlayer = player
	GarrisonLandingPageReport.CharacterName:SetText(selectedPlayer)

	local self = GarrisonLandingPageReport
	self.EmptyString:SetText(L_LOADING);
	self.EmptyString:Show();
	for i = 1, #self.Shipments do
		self.Shipments[i]:Hide();
	end

	inprogressMissions = loadInProgressMissions()

	if ( GarrisonLandingPage:IsShown() ) then
		C_Garrison.RequestLandingPageShipmentInfo();
		GarrisonLandingPageReportList_UpdateItems();
	end

	if ( GarrisonLandingPageReport.selectedTab == GarrisonLandingPageReport.Available ) then
		GarrisonLandingPageReport_SetTab(GarrisonLandingPageReport.InProgress);
	end

	GarrisonLandingPageReport.Available:SetEnabled( isCurrentPlayerSelected() )
end

local function selectCurrentPlayer()
	selectCharacter(getPlayerName())
end

local function selectCharacterFromDropdown(self)
	if ( UIDropDownMenu_GetCurrentDropDown() == MissionMinderCharacterMenu ) then
		CloseDropDownMenus();
	end

	if ( self.value == selectedPlayer ) then
		return
	end

	selectCharacter(self.value)
end

function MissionMinderGarrisonResourceCache_OnEnter(self)
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
		GameTooltip:SetText(GARRISON_CACHE);
		GameTooltip:Show()
end

local function trackMission(event, missionID, ...)
	local player = getPlayerName()
	local now = time()

	local mission = nil

	-- Find this mission and save its details
	GarrisonMissionList_UpdateMissions()
	local missions = GarrisonMissionFrame.MissionTab.MissionList.availableMissions --C_Garrison.GetLandingPageItems()
	for i=1, #missions do
		if ( missionID == missions[i].missionID ) then
			mission = {}
			mission.missionID = missions[i].missionID
			mission.level = missions[i].level
			mission.iLevel = missions[i].iLevel
			mission.name = missions[i].name
			mission.type = missions[i].type
			mission.typeAtlas = missions[i].typeAtlas
			mission.location = missions[i].location
			mission.locPrefix = missions[i].locPrefix
			mission.isRare = missions[i].isRare

			mission.numFollowers = missions[i].numFollowers
			mission.followers = {}
			for ii = 1, #missions[i].followers do
				local followerID = missions[i].followers[ii]
				tinsert(mission.followers, C_Garrison.GetFollowerName(followerID))
			end

			mission.numRewards = missions[i].numRewards
			mission.rewards = {}
			for k,reward in pairs(missions[i].rewards) do
				if ( reward.quality ) then
					tinsert(mission.rewards, ITEM_QUALITY_COLORS[reward.quality + 1].hex..reward.title..FONT_COLOR_CODE_CLOSE)
				elseif ( reward.itemID ) then
					local itemName, _, itemRarity = GetItemInfo(reward.itemID);
					tinsert(mission.rewards, ITEM_QUALITY_COLORS[itemRarity].hex..itemName..FONT_COLOR_CODE_CLOSE);
				else
					tinsert(mission.rewards, reward.title)
				end
			end

			break;
		end
	end

	local _, totalTimeInSeconds = C_Garrison.GetPartyMissionInfo(missionID);
	if (mission and totalTimeInSeconds) then
		mission.endDate = now + totalTimeInSeconds
	end


	-- Save the mission to globals
	missions = PreLoadLib.Globals.Characters[player].missions
	if ( not missions ) then
		missions = {}
		PreLoadLib.Globals.Characters[player].missions = missions
	end

	missions[missionID] = mission
end

local function untrackMission(event, missionID)
	local player = getPlayerName()

	local missions = PreLoadLib.Globals.Characters[player].missions
	if ( missions ) then
		missions[missionID] = nil
	end
end

local function isChecked(self)
	return self.value == selectedPlayer
end

function MissionMinderChangeMenu_Toggle()
	ToggleDropDownMenu(1, nil, GarrisonLandingPageReport.CharacterMenu, GarrisonLandingPageReport.ChangeCharacter, 0, 0);
end

function MissionMinderCharacterMenuDropDown_Initialize()
	local currentPlayer = getPlayerName()
	local info = UIDropDownMenu_CreateInfo();
	info.isNotRadio = true;

	--info.text = "Characters"
	--info.isTitle = true
	--UIDropDownMenu_AddButton(info);

	info.isTitle = false
	info.text = currentPlayer
	info.checked = isChecked
	info.value = currentPlayer
	info.func = selectCharacterFromDropdown
	UIDropDownMenu_AddButton(info)

	for character, _ in pairs(PreLoadLib.Globals.Characters) do
		if ( character ~= currentPlayer) then
			info.text = character
			info.value = character
			UIDropDownMenu_AddButton(info)
		end
	end
end

local function updateInProgressMissionsForSelectedCharacter(...)
	if ( isCurrentPlayerSelected() ) then
		module.GarrisonLandingPageReportList_Update(...)
		return;
	end

	local now = time()

	local items = inprogressMissions or {}
	local numItems = #items;
	local scrollFrame = GarrisonLandingPageReport.List.listScroll;
	local buttons = scrollFrame.buttons;
	local offset = HybridScrollFrame_GetOffset(scrollFrame);

	if (numItems == 0) then
		GarrisonLandingPageReport.List.EmptyMissionText:SetText(GARRISON_EMPTY_IN_PROGRESS_LIST);
	else
		GarrisonLandingPageReport.List.EmptyMissionText:SetText(nil);
	end

	for i = 1, #buttons do
		local button = buttons[i];
		local index = offset + i; -- adjust index
		local item = items[index];

		if ( item ) then
			button.id = index;
			local bgName;
			if (item.isBuilding) then
				bgName = "GarrLanding-Building-";
				button.Status:SetText(GARRISON_LANDING_STATUS_BUILDING);
			else
				bgName = "GarrLanding-Mission-";
			end
			button.Title:SetText(item.name);

			local isComplete = (now >= item.endDate)
			if ( isComplete ) then
				bgName = bgName.."Complete";
				button.MissionType:SetText(GARRISON_LANDING_BUILDING_COMPLEATE);
				button.MissionType:SetTextColor(YELLOW_FONT_COLOR.r, YELLOW_FONT_COLOR.g, YELLOW_FONT_COLOR.b);
				button.Title:SetWidth(290);
			else
				bgName = bgName.."InProgress";
				button.MissionType:SetTextColor(GARRISON_MISSION_TYPE_FONT_COLOR.r, GARRISON_MISSION_TYPE_FONT_COLOR.g, GARRISON_MISSION_TYPE_FONT_COLOR.b);
				if (item.isBuilding) then
					button.MissionType:SetText(GARRISON_BUILDING_IN_PROGRESS);
				else
					button.MissionType:SetText(item.type);
				end
				button.TimeLeft:SetText(PreLoadLib.tracker.secondsToString(item.endDate - now));
				button.Title:SetWidth(322 - button.TimeLeft:GetWidth());
			end
			button.MissionTypeIcon:SetAtlas(item.typeAtlas);
			button.MissionTypeIcon:SetShown(not item.isBuilding);
			button.Status:SetShown(not isComplete);
			button.TimeLeft:SetShown(not isComplete);

			button.BG:SetAtlas(bgName, true);
			for i = 1, #button.Rewards do
				button.Rewards[i]:Hide();
			end
			button:Show();
		else
			button:Hide();
		end
	end
end

local function updateAvailableMissionsForSelectedCharacter(...)
	if ( isCurrentPlayerSelected() ) then
		module.GarrisonLandingPageReportList_UpdateAvailable(...)
		return;
	end

	local items = GarrisonLandingPageReport.List.AvailableItems or {};
	local numItems = #items;
	local scrollFrame = GarrisonLandingPageReport.List.listScroll;
	local buttons = scrollFrame.buttons;
	local numButtons = #buttons;

	for i = 1, numButtons do
		local button = buttons[i];
		button:Hide();
	end

	GarrisonLandingPageReport.List.EmptyMissionText:SetText(GARRISON_EMPTY_MISSION_LIST);
end

local function updateMissionsForSelectedCharacter()
	if ( isCurrentPlayerSelected() ) then
		return;
	end

	local self = GarrisonLandingPageReport
	self.InProgress.Text:SetFormattedText(GARRISON_LANDING_IN_PROGRESS, inprogressMissions and #inprogressMissions or 0);
	self.Available.Text:SetFormattedText(GARRISON_LANDING_AVAILABLE, 0);
end

local function updateShipmentsForSelectedCharacter(self)
	local cacheAmount = PreLoadLib.tracker.estimateGarrisonCache(selectedPlayer);
	if ( cacheAmount ) then
		self.GarrisonResourceCache.Amount:SetText(cacheAmount);
		self.GarrisonResourceCache:Show();
	else
		self.GarrisonResourceCache:Hide();
	end

	if ( isCurrentPlayerSelected() ) then
		self.EmptyString:Hide();
	else
		self.EmptyString:SetText(L_NO_DATA_AVAILABLE);
		self.EmptyString:Show();

		for i = 1, #self.Shipments do
			self.Shipments[i]:Hide();
		end
	end
end

local function initialize()
	DEBUG("   |cffffffffTracker|r initialized");

	-- Create CharacterName fontstring
	GarrisonLandingPageReport.CharacterName = GarrisonLandingPageReport:CreateFontString(nil, "BORDER", "GameFontNormal")
	GarrisonLandingPageReport.CharacterName:SetPoint("TOPLEFT", GarrisonLandingPage.HeaderBar, "BOTTOMLEFT", 26, 13)
	GarrisonLandingPageReport.CharacterName:Show()

	-- Reset seleccted player on LandingPage close
	GarrisonLandingPage:HookScript("OnHide", selectCurrentPlayer)

	-- Hide dropdown on LandingPage clsoe
	GarrisonLandingPageReport:HookScript("OnHide", function()
		if ( UIDropDownMenu_GetCurrentDropDown() == MissionMinderCharacterMenu ) then
			CloseDropDownMenus();
		end
	end);

	-- Track mission progress across multiple characters
	Lib.RegisterCallback("tracker", "MISSION_STARTED", trackMission)
	Lib.RegisterCallback("tracker", "MISSION_COMPLETED", untrackMission)

	GarrisonLandingPageReport.EmptyString = GarrisonLandingPageReport:CreateFontString(nil, "BORDER", "GameFontHighlightLarge")
	GarrisonLandingPageReport.EmptyString:SetPoint("BOTTOMLEFT", 60, 40)
	GarrisonLandingPageReport.EmptyString:SetTextColor(1, 0.82, 0)
	GarrisonLandingPageReport.EmptyString:Hide()

	-- Update Shipments for the selected character
	hooksecurefunc("GarrisonLandingPageReport_GetShipments", updateShipmentsForSelectedCharacter)

	-- Update missions for the selected character
	hooksecurefunc("GarrisonLandingPageReportList_UpdateItems", updateMissionsForSelectedCharacter)
	module.GarrisonLandingPageReportList_UpdateAvailable = GarrisonLandingPageReportList_UpdateAvailable
	GarrisonLandingPageReportList_UpdateAvailable = updateAvailableMissionsForSelectedCharacter
	module.GarrisonLandingPageReportList_Update = GarrisonLandingPageReportList_Update
	GarrisonLandingPageReportList_Update = updateInProgressMissionsForSelectedCharacter

	-- Set the default player to the current player
	selectCurrentPlayer()
end

local events = Lib.events
events:RegisterEvent("tracker", "ADDON_LOADED", initialize);
